/*****************************************************************************/
/*                        WiNRADiO G39DDC EXCELSIOR                          */
/*                                SDK DEMO                                   */
/*             Simple playback on waveform-audio output device               */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#include "u_waveout.h"

TWaveOut::TWaveOut(void)
{
    m_PaSimple=NULL;
    pthread_mutex_init(&m_Mutex,NULL);
}

TWaveOut::~TWaveOut(void)
{
    Close();

    pthread_mutex_destroy(&m_Mutex);
}

void TWaveOut::Close(void)
{
 pa_simple *PaSimple;

    if(!m_PaSimple)
    {
        return;
    }

    PaSimple=m_PaSimple;

    pthread_mutex_lock(&m_Mutex);
    m_PaSimple=NULL;
    pthread_mutex_unlock(&m_Mutex);

    pa_simple_free(PaSimple);
}

bool TWaveOut::Open(uint32_t SampleRate)
{
 pa_sample_spec SampleFormat;
 pa_simple *PaSimple;
 int Error;

    Close();

    SampleFormat.channels=2;
    SampleFormat.rate=SampleRate;
    SampleFormat.format=PA_SAMPLE_FLOAT32LE;

    PaSimple=pa_simple_new(NULL,"G39DDC SDK Demo",PA_STREAM_PLAYBACK,NULL,"RX",&SampleFormat,NULL,NULL,&Error);

    if(PaSimple==NULL)
    {
        return false;
    }

    m_PaSimple=PaSimple;

    return true;
}

void TWaveOut::Write(const float *Samples,uint32_t Count)
{
 int Error;

    pthread_mutex_lock(&m_Mutex);

    if(m_PaSimple)
    {
        pa_simple_write(m_PaSimple,Samples,Count*2*sizeof(float),&Error);
    }

    pthread_mutex_unlock(&m_Mutex);
}
