/* ***********************************************************************/
/*                                                                       */
/*                      WiNRADiO G35DDC Excalibur Ultra                  */
/*         Header file for G35DDC device driver (PCIe)                   */
/*                     (C) Copyright 2014 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G39DDC_H__
#define __G39DDC_H__

#ifdef HAVE_CONFIG_H
#include <linux/config.h>
#endif

#define __G35_USB__
#define __G35_PCI__
#define __G35_DEBUG__

#include <linux/version.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/time.h>
#include <linux/mm.h>
#include <linux/vmalloc.h>
#include <linux/kthread.h>
#include <linux/timer.h>

#ifdef CONFIG_COMPAT
#include <linux/compat.h>
#endif //CONFIG_COMPAT

#ifdef __G35_USB__
#include <linux/usb.h>
#endif //__G35_USB__

#ifdef __G35_PCI__
#include <linux/pci.h>
#include <linux/interrupt.h>
#endif //__G35_PCI__

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) ((a<<16)|(b<<8)|c)
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,11,0)
#include <linux/sched/signal.h>
#endif

#if LINUX_VERSION_CODE<KERNEL_VERSION(2,6,27)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
typedef unsigned long uintptr_t;
#endif

#include "g35ddc_ioctl.h"
#include "g35ddc_ioctl_compat.h"

#define G35_DEVICE_NAME             "g35ddc"
#define G35_MAX_DEVICE_COUNT        32
#define G35_DESCRIPTION             "WiNRADiO G35DDC Device Driver"
#define G35_DDC1_COUNT              2

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)

#define g35_mutex                   mutex
#define g35_mutex_init(__m__)       mutex_init(__m__)
#define g35_mutex_destroy(__m__)    mutex_destroy(__m__)
#define g35_mutex_lock(__m__)       mutex_lock(__m__)
#define g35_mutex_unlock(__m__)     mutex_unlock(__m__)

#else

#define g35_mutex                   semaphore
#define g35_mutex_init(__m__)       sema_init(__m__,1)
#define g35_mutex_destroy(__m__)
#define g35_mutex_lock(__m__)       down(__m__)
#define g35_mutex_unlock(__m__)     up(__m__)

#endif

#pragma pack(push,1)

struct g35_io_buffer
{
    struct list_head        list_entry;
    uintptr_t               user_buffer;
    uint32_t                size;
    int                     page_count;
    struct page             **pages;
    int                     changed;

#ifdef __G35_USB__
    void                    *mem;
#endif //__G35_USB__

#ifdef __G35_PCI__
    struct scatterlist      *sg_list;
    int                     sg_segment_count;
    struct pci_dev          *sg_pci_dev;
    enum dma_data_direction sg_direction;
#endif //__G35_PCI__
};

struct g35_complete_io_buffers
{
    struct list_head    list;
    struct semaphore    semaphore;
    spinlock_t          spinlock;
};

struct g35_pending_io_buffers
{
    struct list_head    list;
    spinlock_t          spinlock;
};

struct g35_io_stream
{
    uint8_t                         active;
    uint32_t                        buffer_size;
    uint32_t                        bytes_per_sec;
    struct rw_semaphore             rw_semaphore;
    struct g35_pending_io_buffers   pending_buffers;
    struct g35_complete_io_buffers  complete_buffers;
    uint16_t                        id;
};

#pragma pack(pop)

struct g35ddc;

typedef void (*g35_func_delete_t)(struct g35ddc *dev);
typedef int (*g35_func_open_t)(struct g35ddc *dev);
typedef void (*g35_func_close_t)(struct g35ddc *dev);
typedef int (*g35_func_set_power_t)(struct g35ddc *dev,uint32_t power);
typedef int (*g35_func_configure_fpga_t)(struct g35ddc *dev,void __user *user_buffer,uint32_t size);
typedef int (*g35_func_set_noise_blanker_t)(struct g35ddc *dev,uint16_t value);

typedef int (*g35_func_put_buffer_t)(struct g35ddc *dev,uint32_t stream_index,struct g35_io_buffer *io_buffer);

typedef int (*g35_func_if_start_t)(struct g35ddc *dev,uint16_t period);
typedef void (*g35_func_if_stop_t)(struct g35ddc *dev);
typedef int (*g35_func_get_adc_level_t)(struct g35ddc *dev,uint16_t *level);

typedef int (*g35_func_ddc1_prepare_t)(struct g35ddc *dev,uint32_t channel,uint32_t *coeffs,uint32_t num_coeffs);
typedef int (*g35_func_ddc1_start_t)(struct g35ddc *dev,uint32_t channel);
typedef void (*g35_func_ddc1_stop_t)(struct g35ddc *dev,uint32_t channel);
typedef int (*g35_func_ddc1_update_frequency_t)(struct g35ddc *dev,uint32_t channel);
typedef int (*g35_func_ddc1_update_phase_shift_t)(struct g35ddc *dev,uint32_t channel);

typedef int (*g35_func_mmap_t)(struct g35ddc *dev,struct vm_area_struct *vma);

typedef int (*g35_func_rf_control)(struct g35ddc *dev,uint32_t value);

typedef int (*g35_func_usb_control1_t)(struct g35ddc *dev,const uint16_t *params,void __user *user_buffer,uint16_t buffer_size,void __user *transferred);
typedef int (*g35_func_usb_control2_t)(struct g35ddc *dev,void __user *user_buffer,uint16_t buffer_size,void __user *transferred);
typedef int (*g35_func_wd_t)(struct g35ddc *dev);
typedef int (*g35_func_ddc1_playback_start_t)(struct g35ddc *dev,uint32_t *coeffs,uint32_t num_coeffs);
typedef void (*g35_func_ddc1_playback_stop_t)(struct g35ddc *dev);
typedef int (*g35_func_dac_start_t)(struct g35ddc *dev,uint32_t dac_control);
typedef void (*g35_func_dac_stop_t)(struct g35ddc *dev);
typedef void (*g35_func_cancel_buffers_t)(struct g35ddc *dev,int stream_index);

typedef int (*g35_func_coh_arm_t)(struct g35ddc *dev,uint8_t master);
typedef int (*g35_func_coh_trigger_t)(struct g35ddc *dev);
typedef int (*g35_func_coh_set_t)(struct g35ddc *dev,uint8_t value);
typedef int (*g35_func_coh_get_t)(struct g35ddc *dev,uint8_t *value);
typedef int (*g35_func_ddc1_get_timestamp_t)(struct g35ddc *dev,uint32_t channel,union g35_timestamp *timestamp);


struct g35ddc
{
    struct device                   dev;
    struct cdev                     cdev;
    uint32_t                        index;
    struct g35_state                state;
    struct g35_mutex                mutex;
    uint8_t                         dac;

    struct workqueue_struct         *workqueue;
    struct work_struct              wd_work;
    struct timer_list               wd_timer;
    uint32_t                        wd_counter;

    wait_queue_head_t               power_wait_queue;

    uint8_t                         coh_support;

    struct g35_io_stream            io_stream[G35_IO_STREAM_COUNT];
    struct g35_ddc1_params          ddc1_params[G35_DDC1_COUNT];

    g35_func_delete_t               func_delete;
    g35_func_open_t                 func_open;
    g35_func_close_t                func_close;
    g35_func_set_power_t            func_set_power;
    g35_func_configure_fpga_t       func_configure_fpga;

    g35_func_rf_control             func_rf_control1;
    g35_func_rf_control             func_rf_control2;

    g35_func_set_noise_blanker_t    func_set_noise_blanker;

    g35_func_put_buffer_t           func_put_buffer;
    g35_func_if_start_t             func_if_start;
    g35_func_if_stop_t              func_if_stop;

    g35_func_get_adc_level_t        func_get_adc_level;

    g35_func_ddc1_prepare_t         func_ddc1_prepare;
    g35_func_ddc1_start_t           func_ddc1_start;
    g35_func_ddc1_stop_t            func_ddc1_stop;
    g35_func_ddc1_update_frequency_t func_ddc1_update_frequency;
    g35_func_ddc1_update_phase_shift_t func_ddc1_update_phase_shift;

    g35_func_ddc1_playback_start_t  func_ddc1_playback_start;
    g35_func_ddc1_playback_stop_t   func_ddc1_playback_stop;

    g35_func_dac_start_t            func_dac_start;
    g35_func_dac_stop_t             func_dac_stop;

    g35_func_cancel_buffers_t       func_cancel_buffers;

    g35_func_mmap_t                 func_mmap;
    g35_func_wd_t                   func_wd;

    g35_func_coh_set_t              func_coh_set;
    g35_func_coh_get_t              func_coh_get;
    g35_func_coh_arm_t              func_coh_arm;
    g35_func_coh_trigger_t          func_coh_trigger;

    g35_func_ddc1_get_timestamp_t   func_ddc1_get_timestamp;

    uint8_t                         config[G35_CONFIG_SIZE];
};

#define G35_DEV_LOCK    g35_mutex_lock(&dev->mutex)
#define G35_DEV_UNLOCK  g35_mutex_unlock(&dev->mutex)

#define G35_DEV_LOCK2   g35_mutex_lock(&dev->g35_dev.mutex)
#define G35_DEV_UNLOCK2 g35_mutex_unlock(&dev->g35_dev.mutex)

#define G35_REG_LOCK    g35_mutex_lock(&g35_reg_mutex)
#define G35_REG_UNLOCK  g35_mutex_unlock(&g35_reg_mutex)

#define G35_IS_INPUT_STREAM(__index__) ((__index__)>=G35_IO_STREAM_IN_FIRST && (__index__)<=G35_IO_STREAM_IN_LAST)
#define G35_IS_OUTPUT_STREAM(__index__) ((__index__)>=G35_IO_STREAM_OUT_FIRST && (__index__)<=G35_IO_STREAM_OUT_LAST)

#define G35_ERROR(__fmt__,...) printk(KERN_ERR G35_DEVICE_NAME ": " __fmt__,##__VA_ARGS__)

#ifdef __G35_DEBUG__

#define DEBUG(__fmt__,...) printk(KERN_DEBUG G35_DEVICE_NAME ": " __fmt__,##__VA_ARGS__)

#else

#define DEBUG(__fmt__,...)

#endif //__G35_DEBUG__


#ifdef __G35_PCI__

#define G35_PCI_VENDOR                      0x1A39
#define G35_PCI_DEVICE                      0x0004
#define G35_PCI_SUBSYSTEM_VENDOR            0x186F
#define G35_PCI_SUBSYSTEM_DEVICE            0x1205

#define LCLK_DIVIN                          0
#define LCLK_DIVFB                          31
#define LCLK_DIVOUT                         7
#define L_PLL_RANGE                         0x3

#define G35_I2C_DIV_A                       1
#define G35_I2C_DIV_B                       (((125000000 / (22 * 390000 * (G35_I2C_DIV_A + 1))) - 1)&0x3F)
#define G35_I2C_MAX_RETRY                   3000000
#define G35_I2C_FIFO_SIZE                   8

#define PCI_SYS_CFG_SYSTEM                  0x800
#define LB_CTL                              0x804
#define CLK_CSR                             0x808
#define INT_STAT                            0x814
#define INT_CFG0                            0x820
#define INT_CFG1                            0x824
#define INT_CFG2                            0x828
#define INT_CFG3                            0x82C

#define TWI_CTRL                            0x900
#define TWI_STATUS                          0x904
#define TWI_ADDRESS                         0x908
#define TWI_DATA                            0x90C
#define TWI_IRT_STATUS                      0x910
#define TWI_TR_SIZE                         0x914
#define TWI_TO                              0x91C
#define TWI_IR_EN                   0x924
#define TWI_IR_DIS                  0x928


#define GPIO_DIRECTION_MODE                 0xA04
#define GPIO_OUTPUT_ENABLE                  0xA08
#define GPIO_OUTPUT_VALUE                   0xA0C
#define GPIO_INPUT_VALUE                    0xA10

#define GPIO_INT_MASK_CLR                   0xA18
#define GPIO_INT_MASK_SET                   0xA1C
#define GPIO_INT_STATUS                     0xA20
#define GPIO_INT_TYPE                       0xA24
#define GPIO_INT_VALUE                      0xA28


#define FCL_CTRL                            0xB00
#define FCL_STATUS                          0xB04
#define FCL_IODATA_IN                       0xB08
#define FCL_IODATA_OUT                      0xB0C
#define FCL_EN                              0xB10
#define FCL_TIMER_0                         0xB14
#define FCL_TIMER_1                         0xB18
#define FCL_CLK_DIV                         0xB1C
#define FCL_IRQ                             0xB20
#define FCL_TIMER_CTRL                      0xB24
#define FCL_IM                              0xB28
#define FCL_TIMER2_0                        0xB2C
#define FCL_TIMER2_1                        0xB30
#define FCL_FIFO_DATA                       0xE00

#define IRQ_I2C                             (1<<0)
#define IRQ_FCL                             (1<<1)
#define IRQ_GPIO                            (1<<15)

#define GPIO_BIT_TMP                        0xFE
#define GPIO_BIT_COH_IN                     (1<<7)
#define GPIO_BIT_FPGA_RESET                 (1<<8)
#define GPIO_BIT_CTL                        (1<<9)
#define GPIO_BIT_TMP_ERROR                  (1<<10)
#define GPIO_BIT_DMA_IRQ                    (1<<11)
#define GPIO_BIT_FAN_ERROR                  (1<<12)
#define GPIO_BIT_COH_OUT                    (1<<12)
#define GPIO_BIT_FE_POWER                   (1<<14)
#define GPIO_BIT_POWER_STATE                (1<<15)


#define REG_RF_CONTROL1             0x0018
#define REG_RF_CONTROL2             0x001C

#define REG_SPI                     0x0020
#define REG_IF_BUFF_CTRL            0x0024
#define REG_ADC_LEVEL               0x0028
#define REG_DATA_LIMIT              0x002C
#define REG_DAC_CONTROL             0x0030
#define REG_DMA_FIFO_RESET          0x003C
#define REG_DMA_CONFIG              0x0040
#define REG_DMA_ENABLE_OUT          0x0044
#define REG_DMA_ENABLE_IN           0x0048
#define REG_DMA_IRQ_STATUS_OUT      0x004C
#define REG_DMA_IRQ_STATUS_IN       0x0050

#define REG_DMA_PTR_OUT_BASE        0x0054
#define REG_DMA_PTR_IN_BASE         (REG_DMA_PTR_OUT_BASE+3*4*4)

#define REG_FIR_COEFF               0x00F4
#define REG_DDC1_CTRL1              0x00F8
#define REG_DDC1_TUNING_WORD        0x00FC
#define REG_DDC1_START_PHASE        0x0100
#define REG_DDC1_CIC_CTRL1          0x0104
#define REG_DDC1_FIR_CTRL1          0x0108

#define REG_DDC1_COUNT              5

#define REG_WDDC_SDRAM_BUFF         0x0120

#define REG_COHERENT_MODE_CTRL      0x0140
#define REG_DDC1_TIMESTAMP_LO       0x0144
#define REG_DDC1_TIMESTAMP_HI       0x0148
#define REG_DDC2_TIMESTAMP_LO       0x014C
#define REG_DDC2_TIMESTAMP_HI       0x0150


#define G35_DMA_IN_IF               0
#define G35_DMA_IN_CH1_DDC1         1
#define G35_DMA_IN_CH2_DDC1         2
#define G35_DMA_IN_CH1_DDC2         3
#define G35_DMA_IN_CH2_DDC2         4

#define G35_DMA_OUT_CH1_DDC1        0
#define G35_DMA_OUT_CH2_DDC1        1
#define G35_DMA_OUT_DAC             2

#define G35_DMA_CHANNEL_OUT_COUNT   G35_IO_STREAM_OUT_COUNT
#define G35_DMA_CHANNEL_IN_COUNT    G35_IO_STREAM_IN_COUNT

#define G35_DMA_PAGE_SIZE           4096

#ifndef DMA_BIT_MASK
#define DMA_BIT_MASK(n)             (((n) == 64)?~0ULL:((1ULL<<(n))-1))
#endif //DMA_BIT_MASK



#pragma pack(push,1)

struct g35_sg_segment_32
{
    uint32_t    address;
    uint32_t    length;
};

struct g35_sg_segment_64
{
    uint64_t    address;
    uint32_t    length;
};

#pragma pack(pop)

struct g35_dma_subchannel
{
    struct g35_io_buffer    *io_buffer;
    dma_addr_t              aligned_sg_mem;
    void                    *aligned_sg_mem_cpu;
    dma_addr_t              sg_mem;
    void                    *sg_mem_cpu;
};

struct g35_dma_channel
{
    uint8_t                     out;
    uint32_t                    index;
    struct g35_dma_subchannel   subchannel[2];
    uint32_t                    subchannel_index;
    struct g35_mutex            mutex;
};

struct g35ddc_pci
{
    struct g35ddc           g35_dev;
    struct pci_dev          *pci_dev;
    uint8_t                 dma_64;
    uint8_t                 *bar0;
    uint8_t                 *bar4;
    uint32_t                dma_status_in;
    uint32_t                dma_subchannel_mask_in;
    uint32_t                dma_status_out;
    uint32_t                dma_subchannel_mask_out;
    uint32_t                dma_enabled_in;
    uint32_t                dma_enabled_out;
    spinlock_t              irq_spinlock;
    struct work_struct      irq_work;
    uint32_t                dma_sg_list_memory_size;
    uint32_t                dma_max_buffer_size;
    uint8_t                 msi;
    uint8_t                 disabled;

    struct g35_dma_channel  dma_channel[G35_IO_STREAM_COUNT];
};

#define bar0_write32(__offset__,__value__) iowrite32(__value__,dev->bar0+(__offset__))
#define bar0_read32(__offset__) ioread32(dev->bar0+(__offset__))
#define bar4_write32(__offset__,__value__) iowrite32(__value__,dev->bar4+(__offset__))
#define bar4_read32(__offset__) ioread32(dev->bar4+(__offset__))

#define gpio_set(__bits__) bar4_write32(GPIO_OUTPUT_VALUE,bar4_read32(GPIO_OUTPUT_VALUE) | (__bits__));bar4_read32(GPIO_OUTPUT_VALUE)
#define gpio_clr(__bits__) bar4_write32(GPIO_OUTPUT_VALUE,bar4_read32(GPIO_OUTPUT_VALUE) & (~(__bits__)));bar4_read32(GPIO_OUTPUT_VALUE)

#endif //__G35_PCI__

#endif //__G39DDC_H__
