/* ***********************************************************************/
/*                                                                       */
/*                      WiNRADiO G35DDC Excalibur Ultra                  */
/*         Ioctl codes and definitions for G35DDC device driver          */
/*                     (C) Copyright 2014 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G35_IOCTL_H__
#define __G35_IOCTL_H__

#define IOCTL_RDX_BASE              0xA00

#define G35_INTERFACE_TYPE_PCIE         0
#define G35_INTERFACE_TYPE_USB          1

#define G35_DDC_NARROW              0
#define G35_DDC_WIDE                1
#define G35_DDC_COUNT               2

#define RECEIVER_MODEL              6
#define DEVICE_EEPROM_SIZE          8192
#define INTERNAL_DRIVER_VERSION     4

#define DEVICE_STATE_ERROR_HIGH_TEMP    0x00000001

#define G35_IO_STREAM_IN_IF             0
#define G35_IO_STREAM_IN_DDC            1
#define G35_IO_STREAM_IN_DDC_WIDE       2
#define G35_IO_STREAM_IN_DDC_PLAYBACK   3
#define G35_IO_STREAM_OUT_DDC_PLAYBACK  4
#define G35_IO_STREAM_OUT_DAC           5

#define G35_IO_STREAM_IN_FIRST          G35_IO_STREAM_IN_IF
#define G35_IO_STREAM_IN_LAST           G35_IO_STREAM_IN_DDC_PLAYBACK
#define G35_IO_STREAM_IN_COUNT          (G35_IO_STREAM_IN_LAST-G35_IO_STREAM_IN_FIRST+1)

#define G35_IO_STREAM_OUT_FIRST         G35_IO_STREAM_OUT_DDC_PLAYBACK
#define G35_IO_STREAM_OUT_LAST          G35_IO_STREAM_OUT_DAC
#define G35_IO_STREAM_OUT_COUNT         (G35_IO_STREAM_OUT_LAST-G35_IO_STREAM_OUT_FIRST+1)

#define G35_IO_STREAM_COUNT             6

#define G35_CONFIG_SIZE                 8196
#define G35_MAX_BUFFER_SIZE             11722752

#pragma pack(push,1)

struct g35_fpga_data
{
    void        *buffer;
    uint32_t    size;
};


union g35_rf_control {
    uint32_t value;
    struct
    {
        uint32_t              external_reference:1;
        uint32_t              lna:1;
        uint32_t              attenuator:3;
        uint32_t              preselectors:8;
        uint32_t              vcxo:1;
        uint32_t              bit:1;
        uint32_t              reserved:17;
    };
};

struct g35_ddc1_params
{
    uint16_t    control_word;
    uint16_t    cic1;
    uint16_t    cic2;
    uint16_t    fir_control;
    uint16_t    fir_len;
    uint32_t    tuning_word;
    uint32_t	phase_shift;
};

struct g35_ddc1_start_params
{
    uint32_t                buffer_size;
    uint32_t                bytes_per_sec;
    struct g35_ddc1_params  params;
    uint32_t                *coeffs;
	uint32_t				num_coeffs;
};

struct g35_dac_params
{
    uint32_t    buffer_size;
    uint32_t    bytes_per_sec;
    uint32_t    dac_control;
};

struct g35_state
{
    uint8_t connected:1;
    uint8_t open:1;
    uint8_t power:1;
    uint8_t suspended:1;
    uint8_t temperature:1;
    uint8_t fpga:1;
};

union g35_timestamp
{
    struct
    {
        uint32_t    lo;
        uint32_t    hi;
    };

    uint64_t        value;
};

#pragma pack(pop)

#define G35_IOCTL_GET_CONFIG                    _IOR(0x8C,0x00,uint8_t[G35_CONFIG_SIZE])
#define G35_IOCTL_GET_STATE                     _IOR(0x8C,0x01,struct g35_state)
#define G35_IOCTL_SET_POWER                     _IOW(0x8C,0x02,uint8_t)
#define G35_IOCTL_CONFIGURE_FPGA                _IOW(0x8C,0x03,struct g35_fpga_data)

#define G35_IOCTL_RF_CONTROL1                   _IOW(0x8c,0x04,union g35_rf_control)
#define G35_IOCTL_RF_CONTROL2                   _IOW(0x8c,0x05,union g35_rf_control)

#define G35_IOCTL_SET_NOISE_BLANKER             _IOW(0x8C,0x06,uint16_t)
#define G35_IOCTL_IF_START                      _IOW(0x8C,0x07,uint16_t)
#define G35_IOCTL_IF_STOP                       _IO(0x8C,0x08)
#define G35_IOCTL_IF_PUT_BUFFER                 _IOW(0x8C,0x09,void*)
#define G35_IOCTL_IF_GET_BUFFER                 _IOR(0x8C,0x0A,void*)
#define G35_IOCTL_GET_ADC_LEVEL                 _IOR(0x8C,0x0B,uint16_t)
#define G35_IOCTL_DDC1_0_PREPARE                _IOW(0x8C,0x0C,struct g35_ddc1_start_params)
#define G35_IOCTL_DDC1_0_STOP                   _IO(0x8C,0x0D)
#define G35_IOCTL_DDC1_0_SET_FREQUENCY          _IOW(0x8C,0x0E,uint32_t)
#define G35_IOCTL_DDC1_0_PUT_BUFFER             _IOW(0x8C,0x0F,void*)
#define G35_IOCTL_DDC1_0_GET_BUFFER             _IOR(0x8C,0x10,void*)
#define G35_IOCTL_DDC1_1_PREPARE                _IOW(0x8C,0x11,struct g35_ddc1_start_params)
#define G35_IOCTL_DDC1_1_STOP                   _IO(0x8C,0x12)
#define G35_IOCTL_DDC1_1_SET_FREQUENCY          _IOW(0x8C,0x13,uint32_t)
#define G35_IOCTL_DDC1_1_PUT_BUFFER             _IOW(0x8C,0x14,void*)
#define G35_IOCTL_DDC1_1_GET_BUFFER             _IOR(0x8C,0x15,void*)
#define G35_IOCTL_DDC1_0_SET_PHASE_SHIFT        _IOW(0x8C,0x16,void*)
#define G35_IOCTL_DDC1_1_SET_PHASE_SHIFT        _IOW(0x8C,0x17,void*)

#define G35_IOCTL_DDC1_0_START                  _IO(0x8C,0x18)
#define G35_IOCTL_DDC1_1_START                  _IO(0x8C,0x19)
#define G35_IOCTL_DDC1_0_GET_TIMESTAMP          _IOR(0x8C,0x1A,union g35_timestamp)
#define G35_IOCTL_DDC1_1_GET_TIMESTAMP          _IOR(0x8C,0x1B,union g35_timestamp)

#define G35_IOCTL_DDC_PLAYBACK_START           _IOW(0x8C,0x30,struct g35_ddc1_start_params)
#define G35_IOCTL_DDC_PLAYBACK_STOP            _IO(0x8C,0x31)
#define G35_IOCTL_DDC_PLAYBACK_PUT_BUFFER_IN   _IOW(0x8C,0x32,void*)
#define G35_IOCTL_DDC_PLAYBACK_GET_BUFFER_IN   _IOR(0x8C,0x33,void*)
#define G35_IOCTL_DDC_PLAYBACK_PUT_BUFFER_OUT  _IOW(0x8C,0x34,void*)
#define G35_IOCTL_DDC_PLAYBACK_GET_BUFFER_OUT  _IOR(0x8C,0x35,void*)
#define G35_IOCTL_DDC_PLAYBACK_SET_FREQUENCY   _IOW(0x8C,0x36,uint32_t)

#define G35_IOCTL_DAC_START                     _IOW(0x8C,0x38,struct g35_dac_params)
#define G35_IOCTL_DAC_STOP                      _IO(0x8C,0x39)
#define G35_IOCTL_DAC_PUT_BUFFER                _IOW(0x8C,0x40,void*)
#define G35_IOCTL_DAC_GET_BUFFER                _IOR(0x8C,0x41,void*)

#define G35_IOCTL_IF_CANCEL_BUFFERS             _IO(0x8C,0x50)
#define G35_IOCTL_DDC1_0_CANCEL_BUFFERS         _IO(0x8C,0x51)
#define G35_IOCTL_DDC1_1_CANCEL_BUFFERS         _IO(0x8C,0x52)
#define G35_IOCTL_CANCEL_BUFFERS_PLAYBACK_IN    _IO(0x8C,0x55)
#define G35_IOCTL_CANCEL_BUFFERS_PLAYBACK_OUT   _IO(0x8C,0x56)

#define G35_IOCTL_WAIT_FOR_POWER_OFF            _IO(0x8C,0x70)

#define G35_IOCTL_COH_SET                       _IOW(0x8C,0x73,uint8_t)
#define G35_IOCTL_COH_GET                       _IOR(0x8C,0x74,uint8_t)
#define G35_IOCTL_COH_ARM                       _IOW(0x8C,0x75,uint8_t)
#define G35_IOCTL_COH_TRIGGER                   _IO(0x8C,0x76)

#endif // __G35_IOCTL_H__
